//version 4.0.2

//section Some commands can be slow
//slide ------------------------------------------------------------------------
//title collapse can be slow

/*txt
{pstd}
for some datasets commands like {cmd:collapse} can be uncomfortably slow
txt*/

//ex
set rmsg on
import delimited using "time_series_covid19_confirmed_global.csv", clear
drop lat v4
collapse (sum) v*, by(countryregion)
set rmsg off
//endex

/*txt
{pstd}
The {cmd:gtools} package (from SSC, by Mauricio Caceres Bravo) contains various commands that are faster than the 
official Stata version. 
txt*/
//ex
set rmsg on
import delimited using "time_series_covid19_confirmed_global.csv", clear
drop lat v4
gcollapse (sum) v*, by(countryregion)
set rmsg off
//endex
//file time_series_covid19_confirmed_global.csv

//endslide ---------------------------------------------------------------------


//slide ------------------------------------------------------------------------
//title reshape can be slow

/*txt
Similarly, {cmd:reshape} can be painfully slow
txt*/
//ex
set rmsg on
import delimited using "time_series_covid19_confirmed_global.csv", clear
drop lat v4
reshape long v, i(provincestate countryregion) j(date)
set rmsg off
//endex

/*txt
{pstd}
{cmd:fastreshape} (available from SSC, by Michael Droste) is faster
txt*/
//ex
set rmsg on
import delimited using "time_series_covid19_confirmed_global.csv", clear
drop lat v4
fastreshape long v, i(provincestate countryregion) j(date)
set rmsg off
//endex

/*txt
{pstd}
{cmd:tolong} (available from SSC, by Rafal Raciborski) is even faster, but less
general
txt*/

//ex
set rmsg on
import delimited using "time_series_covid19_confirmed_global.csv", clear
drop lat v4
tolong v, i(provincestate countryregion) j(date)
set rmsg off
//endex

//endslide ---------------------------------------------------------------------