//version 4.0.2
//section Working with your data

//subsection Workflow

//slide ------------------------------------------------------------------------
//title Workflow


/*txt
{pstd}
An excellent resource for improving ones workflow in Stata is /*cite long09 */

{pstd}
The {stata "ssc desc mkproject":mkproject} package from SSC can help setting up
a new projct:

{pmore} 
It creates the different folders and initial .do files with a commands in them 
that most .do files should have (sometimes called boilerplate code)

{pmore}
It also creates a {help Project Manager:project}, which you can fill with files
so you can have easy access to them.

{pmore}
Additional .do files with the standard commands already in them can be created
witht the {help boilerplate} command, which is part of {cmd:mkproject}

{pstd}
If you typed {cmd:mkproject proj}, it would create the following folders and files:

{cmd}{...}
        proj/
        ├─ proj.stpr
        ├─ admin/
        ├─ docu/
           ├─ research_log.txt
        ├─ posted/
        │  ├─ data/
        ├─ work/
        │  ├─ proj_main.do
        │  ├─ proj_ana01.do
        │  ├─ proj_dta01.do
{txt}{...}

{pstd}
After that it will open the do-file editor with the project open like 
{stata "projmanager proj/proj.stpr": so}

{pstd}
The newest version of {cmd:mkproject} also has a {cmd:git} option that will setup
a git repository.

{pstd}
It als has a {cmd:smclpres} option, for setting up the folder structure and a 
.do file with boilerplate code for a {cmd:smclpres} presentation like this.
txt*/
//dir proj
//file proj/proj.stpr
//dir proj/admin
//dir proj/docu
//file proj/docu/research_log.txt
//dir proj/posted
//dir proj/posted/data
//dir proj/work
//file proj/work/proj_main.do
//file proj/work/proj_ana01.do
//file proj/work/proj_dta01.do

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Settings for your project

/*txt
{pstd}
Larger projects need to be split up over different files

{pstd}
But I often need some settings throughout the project, that could be{p_end}
{pmore}some file names,{p_end}
{pmore}a list of variables, or something like that{p_end}

{pstd}
I want to define them once, but I don't want to use globals

{pstd}
Instead, I typically have one .do file that defines my settings as locals, and use
{cmd:include} at the top of all my other .do files
txt*/

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Between program and .do file

/*txt
{pstd} 
You can run a .do file called child.do by typing {cmd: do child.do}

{pstd}
You can also run that .do file by typing {cmd: do child.do something else}

{pstd}
This will make the following local macros available at the beginning of child.do:

{pmore}
`0' containing: something else

{pmore}
`1' containing: something

{pmore}
`2' containing: else

{pstd}
child.do could do some complicated/fiddly manipulations of a variable, maybe 
multiple levels of {cmd:if} conditions


{pstd}
You want to apply those to multiple variable, say {cmd:var1} and {cmd:var2}

{pmore}
If you were programming you would have put it in a sub-routine.

{pstd}
Now you can create another .do file (e.g. parent.do ) that contains the lines:

{cmd}
        foreach var of varlist var2 var2 {
            do child.do `var'
        }
{txt}

{pstd}
To me this is the ".do file equivalent" of a sub-routine. 

{pstd}
Inside child.do I would start with {cmd:args} in order to give the local macros
`1', `2', etc. more meaningful names
txt*/

//endslide ---------------------------------------------------------------------

//subsection Working with ID variables
//slide ------------------------------------------------------------------------
//title getting at parts of a variable

/*txt
{pstd}
Say we have an ID variable where the eight digit is the continent, the seventh 
and sixth digit the country within that continent, the fifth digit the cite, 
fourth digit the village, and the first three the household within the village.

{pstd}
You want the country, i.e. the last three digits

{pstd}
If I divide by 100,000, then I am getting close; all I need to do is get rid of 
the bit after the decimal point, i.e. round down

{pstd}
That can be done with the {cmd:floor()} function

{pstd}
{cmd:gen country = floor(ID/100000)}
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title typing large numbers

/*txt
{pstd}
If I have to type large numbers, like 100,000, I alway worry that I miscount the zeros

{pstd}
Instead one can also type 1e5

{pstd}
So, for the previous slide I would actually write:

{pstd}
{cmd:gen country = floor(ID/1e5)}
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Even and odd
/*txt
{pstd}
Sometimes I work with data where the odd observations in a unit are the origins 
and the following even observations are the destinations

{pstd}
so I want to find out if a number is even or odd

{pstd}
{cmd:gen origin = mod(sort,2)}
txt*/
//endslide ---------------------------------------------------------------------

//subsection Adding more meta data 

//slide ------------------------------------------------------------------------
//title notes

/*txt
{pstd}
Notes can be attached to the data and to variables

{pstd}
You can add multiple notes to the same variable or dataset

{pstd}
notes can be longer than labels
txt*/

//ex
sysuse auto
note
note foreign : this is american data, so foreign is non-US
note: this is example data that comes with Stata
note
note foreign
//endex

//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title char

/*txt
{pstd}
A char is a more general version of notes

{pstd}
notes are numbered, but chars can have any name
txt*/

//ex
char rep78[Description] repair status
char list rep78[]
//endex

/*txt
{pstd}
For people making data available to others, and who want to add meta data to the 
data there are two options:

{pmore}
They could add the meta data as {cmd:notes}. This makes it probably a bit easier 
for potential users to find that information, but at the same time can interfere
with the user who may also want to use notes.

{pmore}
They could add the meta data as {cmd:chars}. This is less likely to interfere 
with how the user may want to use the data, but makes it a bit harder for the 
user to find that information.
txt*/
//endslide ---------------------------------------------------------------------
